/** 
 * @file llagentpicksinfo.h
 * @brief LLAgentPicksInfo class header file
 *
 * $LicenseInfo:firstyear=2000&license=viewerlgpl$
 * Second Life Viewer Source Code
 * Copyright (C) 2010, Linden Research, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License only.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * Linden Research, Inc., 945 Battery Street, San Francisco, CA  94111  USA
 * $/LicenseInfo$
 */

#ifndef LL_LLAGENTPICKS_H
#define LL_LLAGENTPICKS_H

#include "llsingleton.h"

struct LLAvatarPicks;

/**
 * Class that provides information about Agent Picks
 */
class LLAgentPicksInfo : public LLSingleton<LLAgentPicksInfo>
{
	class LLAgentPicksObserver;

public:

	LLAgentPicksInfo();
	
	virtual ~LLAgentPicksInfo();

	/**
	 * Requests number of picks from server. 
	 * 
	 * Number of Picks is requested from server, thus it is not available immediately.
	 */
	void requestNumberOfPicks();

	/**
	 * Returns number of Picks.
	 */
	S32 getNumberOfPicks() { return mNumberOfPicks; }

	/**
	 * Returns maximum number of Picks.
	 */
	S32 getMaxNumberOfPicks() { return mMaxNumberOfPicks; }

	/**
	 * Returns TRUE if Agent has maximum allowed number of Picks.
	 */
	bool isPickLimitReached();

	/**
	 * After creating or deleting a Pick we can assume operation on server will be 
	 * completed successfully. Incrementing/decrementing number of picks makes new number
	 * of picks available immediately. Actual number of picks will be updated when we receive 
	 * response from server.
	 */
	void incrementNumberOfPicks() { ++mNumberOfPicks; }

	void decrementNumberOfPicks() { --mNumberOfPicks; }

private:

	void onServerRespond(LLAvatarPicks* picks);

	/**
	* Sets number of Picks.
	*/
	void setNumberOfPicks(S32 number) { mNumberOfPicks = number; }

	/**
	* Sets maximum number of Picks.
	*/
	void setMaxNumberOfPicks(S32 max_picks) { mMaxNumberOfPicks = max_picks; }

private:

	LLAgentPicksObserver* mAgentPicksObserver;
	S32 mMaxNumberOfPicks;
	S32 mNumberOfPicks;
};

#endif //LL_LLAGENTPICKS_H
